from .__set_paths__ import (
    __data__,
    __gui__,
    __home__,
    __icon__,
    __providers__,
    __tabs__,
    __titlebar__,
    __utils__,
)
from .__set_video__ import get_video_file_data
from .__version__ import __version__

__video__ = get_video_file_data()
